@@ -2,9 +2,7 @@
 # Licensed under the MIT License.
 
 import os
-import re
 import shutil
-import warnings
 import pandas as pd
 import gzip
 import random
@@ -12,13 +10,26 @@ import logging
 import _pickle as cPickle
 
 from recommenders.utils.constants import SEED
-from recommenders.datasets.download_utils import maybe_download, download_path
+from recommenders.datasets.download_utils import maybe_download
 
 
 random.seed(SEED)
 logger = logging.getLogger()
 
 
+def get_review_data(reviews_file):
+    """Downloads amazon review data (only), prepares in the required format
+    and stores in the same location
+
+    Args:
+        reviews_file (str): Filename for downloaded reviews dataset.
+    """
+    reviews_name = reviews_file.split("/")[-1]  # *.json (for url)
+    download_and_extract(reviews_name, reviews_file)
+    reviews_output = _reviews_preprocessing(reviews_file)
+    return reviews_output
+
+
 def data_preprocessing(
     reviews_file,
     meta_file,
@@ -287,15 +298,19 @@ def _data_generating_no_history_expanding(
             fo = f_test
         if user_id != last_user_id or tfile == "valid" or tfile == "test":
             if last_user_id is not None:
-                history_clk_num = len(movie_id_list)
+                history_clk_num = len(
+                    movie_id_list  # noqa: F821 undefined name 'movie_id_list'
+                )
                 cat_str = ""
                 mid_str = ""
                 dt_str = ""
-                for c1 in cate_list[:-1]:
+                for c1 in cate_list[:-1]:  # noqa: F821 undefined name 'cate_list'
                     cat_str += c1 + ","
-                for mid in movie_id_list[:-1]:
+                for mid in movie_id_list[  # noqa: F821 undefined name 'movie_id_list'
+                    :-1
+                ]:
                     mid_str += mid + ","
-                for dt_time in dt_list[:-1]:
+                for dt_time in dt_list[:-1]:  # noqa: F821 undefined name 'dt_list'
                     dt_str += dt_time + ","
                 if len(cat_str) > 0:
                     cat_str = cat_str[:-1]
@@ -322,7 +337,7 @@ def _data_generating_no_history_expanding(
                         + dt_str
                         + "\n"
                     )
-            if tfile == "train" or last_user_id == None:
+            if tfile == "train" or last_user_id is None:
                 movie_id_list = []
                 cate_list = []
                 dt_list = []
