@@ -1,7 +1,6 @@
 # Copyright (c) Microsoft Corporation. All rights reserved.
 # Licensed under the MIT License.
 
-
 import numpy as np
 
 try:
@@ -102,13 +101,13 @@ class SparkRatingEvaluation:
             raise ValueError("Schema of rating_pred not valid. Missing Prediction Col")
 
         self.rating_true = self.rating_true.select(
-            col(self.col_user).cast("double"),
-            col(self.col_item).cast("double"),
+            col(self.col_user),
+            col(self.col_item),
             col(self.col_rating).cast("double").alias("label"),
         )
         self.rating_pred = self.rating_pred.select(
-            col(self.col_user).cast("double"),
-            col(self.col_item).cast("double"),
+            col(self.col_user),
+            col(self.col_item),
             col(self.col_prediction).cast("double").alias("prediction"),
         )
 
@@ -161,7 +160,8 @@ class SparkRatingEvaluation:
             0
         ]
         var2 = self.y_pred_true.selectExpr("variance(label)").collect()[0][0]
-        return 1 - var1 / var2
+        # numpy divide is more tolerant to var2 being zero
+        return 1 - np.divide(var1, var2)
 
 
 class SparkRankingEvaluation:
@@ -323,9 +323,13 @@ class SparkRankingEvaluation:
         Return:
             float: recall at k (min=0, max=1).
         """
-        recall = self._items_for_user_all.rdd.map(
-            lambda x: float(len(set(x[0]).intersection(set(x[1])))) / float(len(x[1]))
-        ).mean()
+        df_hit = self._items_for_user_all.withColumn(
+            "hit", F.array_intersect(DEFAULT_PREDICTION_COL, "ground_truth")
+        )
+        df_hit = df_hit.withColumn("num_hit", F.size("hit"))
+        df_hit = df_hit.withColumn("num_actual", F.size("ground_truth"))
+        df_hit = df_hit.withColumn("per_hit", df_hit["num_hit"] / df_hit["num_actual"])
+        recall = df_hit.select(F.mean("per_hit")).collect()[0][0]
 
         return recall
 
@@ -510,7 +514,8 @@ class SparkDiversityEvaluation:
                recommendations to all users.
         * Novelty - A more novel item indicates it is less popular, i.e. it gets recommended less frequently.
         * Diversity - The dissimilarity of items being recommended.
-        * Serendipity - The "unusualness" or "surprise" of recommendations to a user. When 'col_relevance' is used, it indicates how "pleasant surprise" of recommendations is to a user.
+        * Serendipity - The "unusualness" or "surprise" of recommendations to a user. When 'col_relevance' is used,
+            it indicates how "pleasant surprise" of recommendations is to a user.
 
         The metric definitions/formulations are based on the following references with modification:
 
@@ -534,8 +539,10 @@ class SparkDiversityEvaluation:
                 Interaction here follows the *item choice model* from Castells et al.
             reco_df (pyspark.sql.DataFrame): Recommender's prediction output, containing col_user, col_item,
                 col_relevance (optional). Assumed to not contain any duplicate user-item pairs.
-            item_feature_df (pyspark.sql.DataFrame): (Optional) It is required only when item_sim_measure='item_feature_vector'. It contains two columns: col_item and features (a feature vector).
-            item_sim_measure (str): (Optional) This column indicates which item similarity measure to be used. Available measures include item_cooccurrence_count (default choice) and item_feature_vector.
+            item_feature_df (pyspark.sql.DataFrame): (Optional) It is required only when item_sim_measure='item_feature_vector'.
+                It contains two columns: col_item and features (a feature vector).
+            item_sim_measure (str): (Optional) This column indicates which item similarity measure to be used.
+                Available measures include item_cooccurrence_count (default choice) and item_feature_vector.
             col_user (str): User id column name.
             col_item (str): Item id column name.
             col_relevance (str): Optional. This column indicates whether the recommended item is actually
@@ -587,8 +594,9 @@ class SparkDiversityEvaluation:
                     )
             else:
                 raise Exception(
-                    "item_feature_df not specified! item_feature_df must be provided if choosing to use item_feature_vector to calculate item similarity. item_feature_df should have schema:"
-                    + str(required_schema)
+                    "item_feature_df not specified! item_feature_df must be provided "
+                    "if choosing to use item_feature_vector to calculate item similarity. "
+                    "item_feature_df should have schema:" + str(required_schema)
                 )
 
         # check if reco_df contains any user_item pairs that are already shown in train_df
