@@ -13,9 +13,7 @@ from sklearn.metrics import (
 import numpy as np
 import yaml
 import zipfile
-import json
 import pickle as pkl
-import tensorflow as tf
 
 from recommenders.datasets.download_utils import maybe_download
 
@@ -300,10 +298,47 @@ def load_yaml(filename):
         return config
     except FileNotFoundError:  # for file not found
         raise
-    except Exception as e:  # for other exceptions
+    except Exception:  # for other exceptions
         raise IOError("load {0} error!".format(filename))
 
 
+class HParams:
+    """Class for holding hyperparameters for DeepRec algorithms."""
+
+    def __init__(self, hparams_dict):
+        """Create an HParams object from a dictionary of hyperparameter values.
+
+        Args:
+            hparams_dict (dict): Dictionary with the model hyperparameters.
+        """
+        for val in hparams_dict.values():
+            if not (
+                isinstance(val, int)
+                or isinstance(val, float)
+                or isinstance(val, str)
+                or isinstance(val, list)
+            ):
+                raise ValueError(
+                    "Hyperparameter value {} should be integer, float, string or list.".format(
+                        val
+                    )
+                )
+        self._values = hparams_dict
+        for hparam in hparams_dict:
+            setattr(self, hparam, hparams_dict[hparam])
+
+    def __repr__(self):
+        return "HParams object with values {}".format(self._values.__repr__())
+
+    def values(self):
+        """Return the hyperparameter values as a dictionary.
+
+        Returns:
+            dict: Dictionary with the hyperparameter values.
+        """
+        return self._values
+
+
 def create_hparams(flags):
     """Create the model hyperparameters.
 
@@ -311,167 +346,62 @@ def create_hparams(flags):
         flags (dict): Dictionary with the model requirements.
 
     Returns:
-        tf.contrib.training.HParams: Hyperparameter object in TF.
+        HParams: Hyperparameter object.
     """
-    return tf.contrib.training.HParams(
-        # data
-        kg_file=flags["kg_file"] if "kg_file" in flags else None,
-        user_clicks=flags["user_clicks"] if "user_clicks" in flags else None,
-        FEATURE_COUNT=flags["FEATURE_COUNT"] if "FEATURE_COUNT" in flags else None,
-        FIELD_COUNT=flags["FIELD_COUNT"] if "FIELD_COUNT" in flags else None,
-        data_format=flags["data_format"] if "data_format" in flags else None,
-        PAIR_NUM=flags["PAIR_NUM"] if "PAIR_NUM" in flags else None,
-        DNN_FIELD_NUM=flags["DNN_FIELD_NUM"] if "DNN_FIELD_NUM" in flags else None,
-        n_user=flags["n_user"] if "n_user" in flags else None,
-        n_item=flags["n_item"] if "n_item" in flags else None,
-        n_user_attr=flags["n_user_attr"] if "n_user_attr" in flags else None,
-        n_item_attr=flags["n_item_attr"] if "n_item_attr" in flags else None,
-        iterator_type=flags["iterator_type"] if "iterator_type" in flags else None,
-        SUMMARIES_DIR=flags["SUMMARIES_DIR"] if "SUMMARIES_DIR" in flags else None,
-        MODEL_DIR=flags["MODEL_DIR"] if "MODEL_DIR" in flags else None,
+    init_dict = {
         # dkn
-        wordEmb_file=flags["wordEmb_file"] if "wordEmb_file" in flags else None,
-        entityEmb_file=flags["entityEmb_file"] if "entityEmb_file" in flags else None,
-        contextEmb_file=flags["contextEmb_file"]
-        if "contextEmb_file" in flags
-        else None,
-        news_feature_file=flags["news_feature_file"]
-        if "news_feature_file" in flags
-        else None,
-        user_history_file=flags["user_history_file"]
-        if "user_history_file" in flags
-        else None,
-        use_entity=flags["use_entity"] if "use_entity" in flags else True,
-        use_context=flags["use_context"] if "use_context" in flags else True,
-        doc_size=flags["doc_size"] if "doc_size" in flags else None,
-        history_size=flags["history_size"] if "history_size" in flags else None,
-        word_size=flags["word_size"] if "word_size" in flags else None,
-        entity_size=flags["entity_size"] if "entity_size" in flags else None,
-        entity_dim=flags["entity_dim"] if "entity_dim" in flags else None,
-        entity_embedding_method=flags["entity_embedding_method"]
-        if "entity_embedding_method" in flags
-        else None,
-        transform=flags["transform"] if "transform" in flags else None,
-        train_ratio=flags["train_ratio"] if "train_ratio" in flags else None,
+        "use_entity": True,
+        "use_context": True,
         # model
-        dim=flags["dim"] if "dim" in flags else None,
-        layer_sizes=flags["layer_sizes"] if "layer_sizes" in flags else None,
-        cross_layer_sizes=flags["cross_layer_sizes"]
-        if "cross_layer_sizes" in flags
-        else None,
-        cross_layers=flags["cross_layers"] if "cross_layers" in flags else None,
-        activation=flags["activation"] if "activation" in flags else None,
-        cross_activation=flags["cross_activation"]
-        if "cross_activation" in flags
-        else "identity",
-        user_dropout=flags["user_dropout"] if "user_dropout" in flags else False,
-        dropout=flags["dropout"] if "dropout" in flags else [0.0],
-        attention_layer_sizes=flags["attention_layer_sizes"]
-        if "attention_layer_sizes" in flags
-        else None,
-        attention_activation=flags["attention_activation"]
-        if "attention_activation" in flags
-        else None,
-        attention_dropout=flags["attention_dropout"]
-        if "attention_dropout" in flags
-        else 0.0,
-        model_type=flags["model_type"] if "model_type" in flags else None,
-        method=flags["method"] if "method" in flags else None,
-        load_saved_model=flags["load_saved_model"]
-        if "load_saved_model" in flags
-        else False,
-        load_model_name=flags["load_model_name"]
-        if "load_model_name" in flags
-        else None,
-        filter_sizes=flags["filter_sizes"] if "filter_sizes" in flags else None,
-        num_filters=flags["num_filters"] if "num_filters" in flags else None,
-        mu=flags["mu"] if "mu" in flags else None,
-        fast_CIN_d=flags["fast_CIN_d"] if "fast_CIN_d" in flags else 0,
-        use_Linear_part=flags["use_Linear_part"]
-        if "use_Linear_part" in flags
-        else False,
-        use_FM_part=flags["use_FM_part"] if "use_FM_part" in flags else False,
-        use_CIN_part=flags["use_CIN_part"] if "use_CIN_part" in flags else False,
-        use_DNN_part=flags["use_DNN_part"] if "use_DNN_part" in flags else False,
+        "cross_activation": "identity",
+        "user_dropout": False,
+        "dropout": [0.0],
+        "attention_dropout": 0.0,
+        "load_saved_model": False,
+        "fast_CIN_d": 0,
+        "use_Linear_part": False,
+        "use_FM_part": False,
+        "use_CIN_part": False,
+        "use_DNN_part": False,
         # train
-        init_method=flags["init_method"] if "init_method" in flags else "tnormal",
-        init_value=flags["init_value"] if "init_value" in flags else 0.01,
-        embed_l2=flags["embed_l2"] if "embed_l2" in flags else 0.0000,
-        embed_l1=flags["embed_l1"] if "embed_l1" in flags else 0.0000,
-        layer_l2=flags["layer_l2"] if "layer_l2" in flags else 0.0000,
-        layer_l1=flags["layer_l1"] if "layer_l1" in flags else 0.0000,
-        cross_l2=flags["cross_l2"] if "cross_l2" in flags else 0.0000,
-        cross_l1=flags["cross_l1"] if "cross_l1" in flags else 0.0000,
-        reg_kg=flags["reg_kg"] if "reg_kg" in flags else 0.0000,
-        learning_rate=flags["learning_rate"] if "learning_rate" in flags else 0.001,
-        lr_rs=flags["lr_rs"] if "lr_rs" in flags else 1,
-        lr_kg=flags["lr_kg"] if "lr_kg" in flags else 0.5,
-        kg_training_interval=flags["kg_training_interval"]
-        if "kg_training_interval" in flags
-        else 5,
-        max_grad_norm=flags["max_grad_norm"] if "max_grad_norm" in flags else 2,
-        is_clip_norm=flags["is_clip_norm"] if "is_clip_norm" in flags else 0,
-        dtype=flags["dtype"] if "dtype" in flags else 32,
-        loss=flags["loss"] if "loss" in flags else None,
-        optimizer=flags["optimizer"] if "optimizer" in flags else "adam",
-        epochs=flags["epochs"] if "epochs" in flags else 10,
-        batch_size=flags["batch_size"] if "batch_size" in flags else 1,
-        enable_BN=flags["enable_BN"] if "enable_BN" in flags else False,
+        "init_method": "tnormal",
+        "init_value": 0.01,
+        "embed_l2": 0.0,
+        "embed_l1": 0.0,
+        "layer_l2": 0.0,
+        "layer_l1": 0.0,
+        "cross_l2": 0.0,
+        "cross_l1": 0.0,
+        "reg_kg": 0.0,
+        "learning_rate": 0.001,
+        "lr_rs": 1,
+        "lr_kg": 0.5,
+        "kg_training_interval": 5,
+        "max_grad_norm": 2,
+        "is_clip_norm": 0,
+        "dtype": 32,
+        "optimizer": "adam",
+        "epochs": 10,
+        "batch_size": 1,
+        "enable_BN": False,
         # show info
-        show_step=flags["show_step"] if "show_step" in flags else 1,
-        save_model=flags["save_model"] if "save_model" in flags else True,
-        save_epoch=flags["save_epoch"] if "save_epoch" in flags else 5,
-        metrics=flags["metrics"] if "metrics" in flags else None,
-        write_tfevents=flags["write_tfevents"] if "write_tfevents" in flags else False,
+        "show_step": 1,
+        "save_model": True,
+        "save_epoch": 5,
+        "write_tfevents": False,
         # sequential
-        item_embedding_dim=flags["item_embedding_dim"]
-        if "item_embedding_dim" in flags
-        else None,
-        cate_embedding_dim=flags["cate_embedding_dim"]
-        if "cate_embedding_dim" in flags
-        else None,
-        user_embedding_dim=flags["user_embedding_dim"]
-        if "user_embedding_dim" in flags
-        else None,
-        train_num_ngs=flags["train_num_ngs"] if "train_num_ngs" in flags else 4,
-        need_sample=flags["need_sample"] if "need_sample" in flags else True,
-        embedding_dropout=flags["embedding_dropout"]
-        if "embedding_dropout" in flags
-        else 0.0,
-        user_vocab=flags["user_vocab"] if "user_vocab" in flags else None,
-        item_vocab=flags["item_vocab"] if "item_vocab" in flags else None,
-        cate_vocab=flags["cate_vocab"] if "cate_vocab" in flags else None,
-        pairwise_metrics=flags["pairwise_metrics"]
-        if "pairwise_metrics" in flags
-        else None,
-        EARLY_STOP=flags["EARLY_STOP"] if "EARLY_STOP" in flags else 100,
-        # gru4rec
-        max_seq_length=flags["max_seq_length"] if "max_seq_length" in flags else None,
-        hidden_size=flags["hidden_size"] if "hidden_size" in flags else None,
+        "train_num_ngs": 4,
+        "need_sample": True,
+        "embedding_dropout": 0.0,
+        "EARLY_STOP": 100,
         # caser,
-        L=flags["L"] if "L" in flags else None,
-        T=flags["T"] if "T" in flags else None,
-        n_v=flags["n_v"] if "n_v" in flags else None,
-        n_h=flags["n_h"] if "n_h" in flags else None,
-        min_seq_length=flags["min_seq_length"] if "min_seq_length" in flags else 1,
-        # sli_rec
-        attention_size=flags["attention_size"] if "attention_size" in flags else None,
-        att_fcn_layer_sizes=flags["att_fcn_layer_sizes"]
-        if "att_fcn_layer_sizes" in flags
-        else None,
-        # nextitnet
-        dilations=flags["dilations"] if "dilations" in flags else None,
-        kernel_size=flags["kernel_size"] if "kernel_size" in flags else None,
-        # lightgcn
-        embed_size=flags["embed_size"] if "embed_size" in flags else None,
-        n_layers=flags["n_layers"] if "n_layers" in flags else None,
-        decay=flags["decay"] if "decay" in flags else None,
-        eval_epoch=flags["eval_epoch"] if "eval_epoch" in flags else None,
-        top_k=flags["top_k"] if "top_k" in flags else None,
+        "min_seq_length": 1,
         # sum
-        slots=flags["slots"] if "slots" in flags else 5,
-        cell=flags["cell"] if "cell" in flags else "SUM",
-    )
+        "slots": 5,
+        "cell": "SUM",
+    }
+    init_dict.update(flags)
+    return HParams(init_dict)
 
 
 def prepare_hparams(yaml_file=None, **kwargs):
@@ -481,7 +411,7 @@ def prepare_hparams(yaml_file=None, **kwargs):
         yaml_file (str): YAML file as configuration.
 
     Returns:
-        tf.contrib.training.HParams: Hyperparameter object in TF.
+        HParams: Hyperparameter object.
     """
     if yaml_file is not None:
         config = load_yaml(yaml_file)
