@@ -1,4 +1,4 @@
 # Copyright (c) Microsoft Corporation. All rights reserved.
 # Licensed under the MIT License.
 
-from .sar_singlenode import SARSingleNode as SAR
+from .sar_singlenode import SARSingleNode as SAR  # noqa: F401
