@@ -1,15 +1,17 @@
 # Copyright (c) Microsoft Corporation. All rights reserved.
 # Licensed under the MIT License.
-
-
 import numpy as np
 import pandas as pd
 import logging
 from scipy import sparse
 
 from recommenders.utils.python_utils import (
+    cosine_similarity,
+    inclusion_index,
     jaccard,
+    lexicographers_mutual_information,
     lift,
+    mutual_information,
     exponential_decay,
     get_top_k_scored_items,
     rescale,
@@ -17,9 +19,13 @@ from recommenders.utils.python_utils import (
 from recommenders.utils import constants
 
 
-COOCCUR = "cooccurrence"
-JACCARD = "jaccard"
-LIFT = "lift"
+SIM_COOCCUR = "cooccurrence"
+SIM_COSINE = "cosine"
+SIM_INCLUSION_INDEX = "inclusion index"
+SIM_JACCARD = "jaccard"
+SIM_LEXICOGRAPHERS_MUTUAL_INFORMATION = "lexicographers mutual information"
+SIM_LIFT = "lift"
+SIM_MUTUAL_INFORMATION = "mutual information"
 
 logger = logging.getLogger()
 
@@ -40,7 +46,7 @@ class SARSingleNode:
         col_rating=constants.DEFAULT_RATING_COL,
         col_timestamp=constants.DEFAULT_TIMESTAMP_COL,
         col_prediction=constants.DEFAULT_PREDICTION_COL,
-        similarity_type=JACCARD,
+        similarity_type=SIM_JACCARD,
         time_decay_coefficient=30,
         time_now=None,
         timedecay_formula=False,
@@ -55,7 +61,9 @@ class SARSingleNode:
             col_rating (str): rating column name
             col_timestamp (str): timestamp column name
             col_prediction (str): prediction column name
-            similarity_type (str): ['cooccurrence', 'jaccard', 'lift'] option for computing item-item similarity
+            similarity_type (str): ['cooccurrence', 'cosine', 'inclusion index', 'jaccard',
+              'lexicographers mutual information', 'lift', 'mutual information'] option for
+              computing item-item similarity
             time_decay_coefficient (float): number of days till ratings are decayed by 1/2
             time_now (int | None): current time for time decay calculation
             timedecay_formula (bool): flag to apply time decay
@@ -68,9 +76,18 @@ class SARSingleNode:
         self.col_timestamp = col_timestamp
         self.col_prediction = col_prediction
 
-        if similarity_type not in [COOCCUR, JACCARD, LIFT]:
+        available_similarity_types = [
+            SIM_COOCCUR,
+            SIM_COSINE,
+            SIM_INCLUSION_INDEX,
+            SIM_JACCARD,
+            SIM_LIFT,
+            SIM_MUTUAL_INFORMATION,
+            SIM_LEXICOGRAPHERS_MUTUAL_INFORMATION,
+        ]
+        if similarity_type not in available_similarity_types:
             raise ValueError(
-                'Similarity type must be one of ["cooccurrence" | "jaccard" | "lift"]'
+                'Similarity type must be one of ["' + '" | "'.join(available_similarity_types) + '"]'
             )
         self.similarity_type = similarity_type
         self.time_decay_half_life = (
@@ -82,6 +99,7 @@ class SARSingleNode:
         self.user_affinity = None
         self.item_similarity = None
         self.item_frequencies = None
+        self.user_frequencies = None
 
         # threshold - items below this number get set to zero in co-occurrence counts
         if self.threshold <= 0:
@@ -90,6 +108,7 @@ class SARSingleNode:
         # set flag to capture unity-rating user-affinity matrix for scaling scores
         self.normalize = normalize
         self.col_unity_rating = "_unity_rating"
+        self.unity_user_affinity = None
 
         # column for mapping user / item ids to internal indices
         self.col_item_id = "_indexed_items"
@@ -107,8 +126,9 @@ class SARSingleNode:
         self.user2index = None
         self.item2index = None
 
-        # the opposite of the above map - map array index to actual string ID
+        # the opposite of the above maps - map array index to actual string ID
         self.index2item = None
+        self.index2user = None
 
     def compute_affinity_matrix(self, df, rating_col):
         """Affinity matrix.
@@ -156,7 +176,7 @@ class SARSingleNode:
         # group time decayed ratings by user-item and take the sum as the user-item affinity
         return df.groupby([self.col_user, self.col_item]).sum().reset_index()
 
-    def compute_coocurrence_matrix(self, df):
+    def compute_cooccurrence_matrix(self, df):
         """Co-occurrence matrix.
 
         The co-occurrence matrix is defined as :math:`C = U^T * U`
@@ -169,7 +189,6 @@ class SARSingleNode:
         Returns:
             numpy.ndarray: Co-occurrence matrix
         """
-
         user_item_hits = sparse.coo_matrix(
             (np.repeat(1, df.shape[0]), (df[self.col_user_id], df[self.col_item_id])),
             shape=(self.n_users, self.n_items),
@@ -191,12 +210,11 @@ class SARSingleNode:
 
         # generate a map of continuous index values to items
         self.index2item = dict(enumerate(df[self.col_item].unique()))
+        self.index2user = dict(enumerate(df[self.col_user].unique()))
 
-        # invert the mapping from above
+        # invert the mappings from above
         self.item2index = {v: k for k, v in self.index2item.items()}
-
-        # create mapping of users to continuous indices
-        self.user2index = {x[1]: x[0] for x in enumerate(df[self.col_user].unique())}
+        self.user2index = {v: k for k, v in self.index2user.items()}
 
         # set values for the total count of users and items
         self.n_users = len(self.user2index)
@@ -205,9 +223,19 @@ class SARSingleNode:
     def fit(self, df):
         """Main fit method for SAR.
 
+        .. note::
+
+        Please make sure that `df` has no duplicates.
+
         Args:
-            df (pandas.DataFrame): User item rating dataframe
+            df (pandas.DataFrame): User item rating dataframe (without duplicates).
         """
+        select_columns = [self.col_user, self.col_item, self.col_rating]
+        if self.time_decay_flag:
+            select_columns += [self.col_timestamp]
+
+        if df[select_columns].duplicated().any():
+            raise ValueError("There should not be duplicates in the dataframe")
 
         # generate continuous indices if this hasn't been done
         if self.index2item is None:
@@ -218,20 +246,11 @@ class SARSingleNode:
             raise TypeError("Rating column data type must be numeric")
 
         # copy the DataFrame to avoid modification of the input
-        select_columns = [self.col_user, self.col_item, self.col_rating]
-        if self.time_decay_flag:
-            select_columns += [self.col_timestamp]
         temp_df = df[select_columns].copy()
 
         if self.time_decay_flag:
             logger.info("Calculating time-decayed affinities")
             temp_df = self.compute_time_decay(df=temp_df, decay_column=self.col_rating)
-        else:
-            # without time decay use the latest user-item rating in the dataset as the affinity score
-            logger.info("De-duplicating the user-item counts")
-            temp_df = temp_df.drop_duplicates(
-                [self.col_user, self.col_item], keep="last"
-            )
 
         logger.info("Creating index columns")
         # add mapping of user and item ids to indices
@@ -263,27 +282,36 @@ class SARSingleNode:
 
         # calculate item co-occurrence
         logger.info("Calculating item co-occurrence")
-        item_cooccurrence = self.compute_coocurrence_matrix(df=temp_df)
+        item_cooccurrence = self.compute_cooccurrence_matrix(df=temp_df)
 
         # free up some space
         del temp_df
 
+        # creates an array with the frequency of every unique item
         self.item_frequencies = item_cooccurrence.diagonal()
 
         logger.info("Calculating item similarity")
-        if self.similarity_type is COOCCUR:
+        if self.similarity_type == SIM_COOCCUR:
             logger.info("Using co-occurrence based similarity")
             self.item_similarity = item_cooccurrence
-        elif self.similarity_type is JACCARD:
+        elif self.similarity_type == SIM_COSINE:
+            logger.info("Using cosine similarity")
+            self.item_similarity = cosine_similarity(item_cooccurrence)
+        elif self.similarity_type == SIM_INCLUSION_INDEX:
+            logger.info("Using inclusion index")
+            self.item_similarity = inclusion_index(item_cooccurrence)
+        elif self.similarity_type == SIM_JACCARD:
             logger.info("Using jaccard based similarity")
-            self.item_similarity = jaccard(item_cooccurrence).astype(
-                df[self.col_rating].dtype
-            )
-        elif self.similarity_type is LIFT:
+            self.item_similarity = jaccard(item_cooccurrence)
+        elif self.similarity_type == SIM_LEXICOGRAPHERS_MUTUAL_INFORMATION:
+            logger.info("Using lexicographers mutual information similarity")
+            self.item_similarity = lexicographers_mutual_information(item_cooccurrence)
+        elif self.similarity_type == SIM_LIFT:
             logger.info("Using lift based similarity")
-            self.item_similarity = lift(item_cooccurrence).astype(
-                df[self.col_rating].dtype
-            )
+            self.item_similarity = lift(item_cooccurrence)
+        elif self.similarity_type == SIM_MUTUAL_INFORMATION:
+            logger.info("Using mutual information similarity")
+            self.item_similarity = mutual_information(item_cooccurrence)
         else:
             raise ValueError("Unknown similarity type: {}".format(self.similarity_type))
 
@@ -346,27 +374,40 @@ class SARSingleNode:
 
         return test_scores
 
-    def get_popularity_based_topk(self, top_k=10, sort_top_k=True):
+    def get_popularity_based_topk(self, top_k=10, sort_top_k=True, items=True):
         """Get top K most frequently occurring items across all users.
 
         Args:
             top_k (int): number of top items to recommend.
             sort_top_k (bool): flag to sort top k results.
+            items (bool): if false, return most frequent users instead
 
         Returns:
             pandas.DataFrame: top k most popular items.
         """
+        if items:
+            frequencies = self.item_frequencies
+            col = self.col_item
+            idx = self.index2item
+        else:
+            if self.user_frequencies is None:
+                self.user_frequencies = self.user_affinity.getnnz(axis=1).astype(
+                    "int64"
+                )
+            frequencies = self.user_frequencies
+            col = self.col_user
+            idx = self.index2user
 
-        test_scores = np.array([self.item_frequencies])
+        test_scores = np.array([frequencies])
 
         logger.info("Getting top K")
-        top_items, top_scores = get_top_k_scored_items(
+        top_components, top_scores = get_top_k_scored_items(
             scores=test_scores, top_k=top_k, sort_top_k=sort_top_k
         )
 
         return pd.DataFrame(
             {
-                self.col_item: [self.index2item[item] for item in top_items.flatten()],
+                col: [idx[item] for item in top_components.flatten()],
                 self.col_prediction: top_scores.flatten(),
             }
         )
@@ -447,6 +488,35 @@ class SARSingleNode:
         # drop invalid items
         return df.replace(-np.inf, np.nan).dropna()
 
+    def get_topk_most_similar_users(self, user, top_k, sort_top_k=True):
+        """Based on user affinity towards items, calculate the most similar users to the given user.
+
+        Args:
+            user (int): user to retrieve most similar users for
+            top_k (int): number of top items to recommend
+            sort_top_k (bool): flag to sort top k results
+
+        Returns:
+            pandas.DataFrame: top k most similar users and their scores
+        """
+        user_idx = self.user2index[user]
+        similarities = self.user_affinity[user_idx].dot(self.user_affinity.T).toarray()
+        similarities[0, user_idx] = -np.inf
+
+        top_items, top_scores = get_top_k_scored_items(
+            scores=similarities, top_k=top_k, sort_top_k=sort_top_k
+        )
+
+        df = pd.DataFrame(
+            {
+                self.col_user: [self.index2user[user] for user in top_items.flatten()],
+                self.col_prediction: top_scores.flatten(),
+            }
+        )
+
+        # drop invalid items
+        return df.replace(-np.inf, np.nan).dropna()
+
     def recommend_k_items(self, test, top_k=10, sort_top_k=True, remove_seen=False):
         """Recommend top K items for all users which are in the test set
 
