@@ -25,6 +25,7 @@ def test_gpu_vm():
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
     "size, epochs, expected_values, seed",
@@ -64,6 +65,7 @@ def test_ncf_integration(
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
     "size, epochs, batch_size, expected_values, seed",
@@ -118,6 +120,7 @@ def test_ncf_deep_dive_integration(
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
     "size, epochs, expected_values",
@@ -158,6 +161,7 @@ def test_fastai_integration(
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
     "syn_epochs, criteo_epochs, expected_values, seed",
@@ -207,13 +211,14 @@ def test_xdeepfm_integration(
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
     "size, steps, expected_values, seed",
     [
         (
             "1m",
-            50000,
+            10000,
             {
                 "rmse": 0.924958,
                 "mae": 0.741425,
@@ -255,6 +260,7 @@ def test_wide_deep_integration(
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
     "yaml_file, data_path, epochs, batch_size, expected_values, seed",
@@ -306,12 +312,14 @@ def test_slirec_quickstart_integration(
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
-    "epochs, seed, MIND_type, expected_values",
+    "epochs, batch_size, seed, MIND_type, expected_values",
     [
         (
-            8,
+            5,
+            64,
             42,
             "demo",
             {
@@ -326,11 +334,23 @@ def test_slirec_quickstart_integration(
     ],
 )
 def test_nrms_quickstart_integration(
-    notebooks, output_notebook, kernel_name, epochs, seed, MIND_type, expected_values
+    notebooks,
+    output_notebook,
+    kernel_name,
+    epochs,
+    batch_size,
+    seed,
+    MIND_type,
+    expected_values,
 ):
     notebook_path = notebooks["nrms_quickstart"]
 
-    params = {"epochs": epochs, "seed": seed, "MIND_type": MIND_type}
+    params = {
+        "epochs": epochs,
+        "batch_size": batch_size,
+        "seed": seed,
+        "MIND_type": MIND_type,
+    }
     pm.execute_notebook(
         notebook_path, output_notebook, kernel_name=kernel_name, parameters=params
     )
@@ -354,12 +374,14 @@ def test_nrms_quickstart_integration(
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
-    "epochs, seed, MIND_type, expected_values",
+    "epochs, batch_size, seed, MIND_type, expected_values",
     [
         (
-            6,
+            5,
+            64,
             42,
             "demo",
             {
@@ -374,11 +396,23 @@ def test_nrms_quickstart_integration(
     ],
 )
 def test_naml_quickstart_integration(
-    notebooks, output_notebook, kernel_name, epochs, seed, MIND_type, expected_values
+    notebooks,
+    output_notebook,
+    kernel_name,
+    batch_size,
+    epochs,
+    seed,
+    MIND_type,
+    expected_values,
 ):
     notebook_path = notebooks["naml_quickstart"]
 
-    params = {"epochs": epochs, "seed": seed, "MIND_type": MIND_type}
+    params = {
+        "epochs": epochs,
+        "batch_size": batch_size,
+        "seed": seed,
+        "MIND_type": MIND_type,
+    }
     pm.execute_notebook(
         notebook_path, output_notebook, kernel_name=kernel_name, parameters=params
     )
@@ -402,13 +436,15 @@ def test_naml_quickstart_integration(
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
-    "epochs, seed, MIND_type, expected_values",
+    "epochs, batch_size, seed, MIND_type, expected_values",
     [
         (
             5,
-            40,
+            64,
+            42,
             "demo",
             {
                 "res_syn": {
@@ -422,11 +458,23 @@ def test_naml_quickstart_integration(
     ],
 )
 def test_lstur_quickstart_integration(
-    notebooks, output_notebook, kernel_name, epochs, seed, MIND_type, expected_values
+    notebooks,
+    output_notebook,
+    kernel_name,
+    epochs,
+    batch_size,
+    seed,
+    MIND_type,
+    expected_values,
 ):
     notebook_path = notebooks["lstur_quickstart"]
 
-    params = {"epochs": epochs, "seed": seed, "MIND_type": MIND_type}
+    params = {
+        "epochs": epochs,
+        "batch_size": batch_size,
+        "seed": seed,
+        "MIND_type": MIND_type,
+    }
     pm.execute_notebook(
         notebook_path, output_notebook, kernel_name=kernel_name, parameters=params
     )
@@ -450,12 +498,14 @@ def test_lstur_quickstart_integration(
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
-    "epochs, seed, MIND_type, expected_values",
+    "epochs, batch_size, seed, MIND_type, expected_values",
     [
         (
-            6,
+            5,
+            64,
             42,
             "demo",
             {
@@ -470,11 +520,23 @@ def test_lstur_quickstart_integration(
     ],
 )
 def test_npa_quickstart_integration(
-    notebooks, output_notebook, kernel_name, epochs, seed, MIND_type, expected_values
+    notebooks,
+    output_notebook,
+    kernel_name,
+    epochs,
+    batch_size,
+    seed,
+    MIND_type,
+    expected_values,
 ):
     notebook_path = notebooks["npa_quickstart"]
 
-    params = {"epochs": epochs, "seed": seed, "MIND_type": MIND_type}
+    params = {
+        "epochs": epochs,
+        "batch_size": batch_size,
+        "seed": seed,
+        "MIND_type": MIND_type,
+    }
     pm.execute_notebook(
         notebook_path, output_notebook, kernel_name=kernel_name, parameters=params
     )
@@ -498,6 +560,7 @@ def test_npa_quickstart_integration(
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
     "yaml_file, data_path, size, epochs, batch_size, expected_values, seed",
@@ -555,6 +618,7 @@ def test_lightgcn_deep_dive_integration(
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 def test_dkn_quickstart_integration(notebooks, output_notebook, kernel_name):
     notebook_path = notebooks["dkn_quickstart"]
@@ -575,6 +639,7 @@ def test_dkn_quickstart_integration(notebooks, output_notebook, kernel_name):
 
 
 @pytest.mark.gpu
+@pytest.mark.notebooks
 @pytest.mark.integration
 @pytest.mark.parametrize(
     "size, expected_values",
@@ -599,3 +664,62 @@ def test_cornac_bivae_integration(
 
     for key, value in expected_values.items():
         assert results[key] == pytest.approx(value, rel=TOL, abs=ABS_TOL)
+
+
+@pytest.mark.gpu
+@pytest.mark.notebooks
+@pytest.mark.integration
+@pytest.mark.parametrize(
+    "data_dir, num_epochs, batch_size, model_name, expected_values, seed",
+    [
+        (
+            os.path.join("tests", "recsys_data", "RecSys", "SASRec-tf2", "data"),
+            1,
+            128,
+            "sasrec",
+            {"ndcg@10": 0.2626, "Hit@10": 0.4244},
+            42,
+        ),
+        (
+            os.path.join("tests", "recsys_data", "RecSys", "SASRec-tf2", "data"),
+            1,
+            128,
+            "ssept",
+            {"ndcg@10": 0.2626, "Hit@10": 0.4244},
+            42,
+        ),
+    ],
+)
+def test_sasrec_quickstart_integration(
+    notebooks,
+    output_notebook,
+    kernel_name,
+    data_dir,
+    num_epochs,
+    batch_size,
+    model_name,
+    expected_values,
+    seed,
+):
+    notebook_path = notebooks["sasrec_quickstart"]
+    params = {
+        "data_dir": data_dir,
+        "num_epochs": num_epochs,
+        "batch_size": batch_size,
+        "model_name": model_name,
+        "seed": seed,
+    }
+
+    print("Executing notebook ... ")
+    pm.execute_notebook(
+        notebook_path,
+        output_notebook,
+        kernel_name=kernel_name,
+        parameters=params,
+    )
+    results = sb.read_notebook(output_notebook).scraps.dataframe.set_index("name")[
+        "data"
+    ]
+
+    for key, value in expected_values.items():
+        assert results[key] == pytest.approx(value, rel=TOL, abs=ABS_TOL)
